from dataclasses import dataclass

import numpy as np
import xarray as xr

from toargridding.metadata import Variables, get_cf_metadata, Metadata


@dataclass
class Variable:
    var: Variables
    data: np.ndarray
    attributes: dict[str, str]
    encoding: dict[str, str]

    @classmethod
    def from_data(cls, data, variable: Variables, metadata: Metadata | None, **kwargs):
        cf_metadata = get_cf_metadata(variable, metadata=metadata)
        # print(variable.name, cf_metadata)
        return cls(variable, data, **cf_metadata, **kwargs)

    def as_data_array(self, dims):
        da = xr.DataArray(
            self.data,
            name=self.name,
            dims=dims,
            attrs=self.attributes,
        )
        da.encoding = self.encoding
        return da

    @property
    def name(self):
        return self.var.name

    @property
    def size(self):
        return self.data.size

    @property
    def min(self):
        return self.data.min()

    @property
    def max(self):
        return self.data.max()


@dataclass
class Coordinate(Variable):
    step: float | str

    @classmethod
    def from_resolution(
        cls, variable: Variables, resolution: float, min: float, max: float
    ):
        span = max - min
        n = int(span / resolution)  # raise error if invalid inputs ?
        data = np.linspace(min, max, n + 1)

        return cls.from_data(data, variable, None, step=resolution)

    def as_data_array(self):
        return super().as_data_array(dims=self.name)


def get_encoding(variables: list[Variable]):
    return {variable.name: variable.encoding for variable in variables}
